(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@babel/runtime/helpers/esm/extends'), require('@babel/runtime/helpers/esm/objectWithoutPropertiesLoose'), require('react'), require('hoist-non-react-statics'), require('theming'), require('is-in-browser'), require('tiny-warning'), require('jss'), require('symbol-observable'), require('css-vendor'), require('shallow-equal'), require('@emotion/is-prop-valid')) :
  typeof define === 'function' && define.amd ? define(['exports', '@babel/runtime/helpers/esm/extends', '@babel/runtime/helpers/esm/objectWithoutPropertiesLoose', 'react', 'hoist-non-react-statics', 'theming', 'is-in-browser', 'tiny-warning', 'jss', 'symbol-observable', 'css-vendor', 'shallow-equal', '@emotion/is-prop-valid'], factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.reactJss = {}, global._extends, global._objectWithoutPropertiesLoose, global.React, global.hoistNonReactStatics, global.theming, global.isInBrowser, global.warning, global.jss, global.$$observable, global.vendor, global.shallowEqual, global.isPropValid));
})(this, (function (exports, _extends, _objectWithoutPropertiesLoose, React, hoistNonReactStatics, theming, isInBrowser, warning, jss, $$observable, vendor, shallowEqual, isPropValid) { 'use strict';

  function _interopNamespaceDefault(e) {
    var n = Object.create(null);
    if (e) {
      Object.keys(e).forEach(function (k) {
        if (k !== 'default') {
          var d = Object.getOwnPropertyDescriptor(e, k);
          Object.defineProperty(n, k, d.get ? d : {
            enumerable: true,
            get: function () { return e[k]; }
          });
        }
      });
    }
    n.default = e;
    return Object.freeze(n);
  }

  var React__namespace = /*#__PURE__*/_interopNamespaceDefault(React);
  var vendor__namespace = /*#__PURE__*/_interopNamespaceDefault(vendor);

  var getDisplayName = function getDisplayName(Component) {
    return Component.displayName || Component.name || 'Component';
  };
  var getDisplayName$1 = getDisplayName;

  var memoize = function memoize(fn) {
    var lastArgs;
    var lastResult;
    return function () {
      for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }
      if (Array.isArray(lastArgs) && args.length === lastArgs.length) {
        var isSame = true;
        for (var i = 0; i < args.length; i++) {
          if (args[i] !== lastArgs[i]) {
            isSame = false;
          }
        }
        if (isSame) {
          return lastResult;
        }
      }
      lastArgs = args;
      lastResult = fn.apply(void 0, args);
      return lastResult;
    };
  };
  var memoize$1 = memoize;

  var mergeClasses = function mergeClasses(baseClasses, additionalClasses) {
    var combinedClasses = _extends({}, baseClasses);
    for (var name in additionalClasses) {
      combinedClasses[name] = name in combinedClasses ? combinedClasses[name] + " " + additionalClasses[name] : additionalClasses[name];
    }
    return combinedClasses;
  };
  var mergeClasses$1 = mergeClasses;

  /**
   * Global index counter to preserve source order.
   * As we create the style sheet during componentWillMount lifecycle,
   * children are handled after the parents, so the order of style elements would
   * be parent->child. It is a problem though when a parent passes a className
   * which needs to override any childs styles. StyleSheet of the child has a higher
   * specificity, because of the source order.
   * So our solution is to render sheets them in the reverse order child->sheet, so
   * that parent has a higher specificity.
   *
   * We start at [Number.MIN_SAFE_INTEGER] to always insert sheets from react-jss first before any
   * sheet which might be inserted manually by the user.
   */
  var index = Number.MIN_SAFE_INTEGER || -1e9;
  var getSheetIndex = function getSheetIndex() {
    return index++;
  };
  var getSheetIndex$1 = getSheetIndex;

  var JssContext = /*#__PURE__*/React__namespace.createContext({
    classNamePrefix: '',
    disableStylesGeneration: false,
    isSSR: !isInBrowser
  });

  var defaultManagers = new Map();
  var getManager = function getManager(context, managerId) {
    // If `managers` map is present in the context, we use it in order to
    // let JssProvider reset them when new response has to render server-side.
    var managers = context.managers;
    if (managers) {
      if (!managers[managerId]) {
        managers[managerId] = new jss.SheetsManager();
      }
      return managers[managerId];
    }
    var manager = defaultManagers.get(managerId);
    if (!manager) {
      manager = new jss.SheetsManager();
      defaultManagers.set(managerId, manager);
    }
    return manager;
  };
  var manageSheet = function manageSheet(options) {
    var sheet = options.sheet,
      context = options.context,
      index = options.index,
      theme = options.theme;
    if (!sheet) {
      return;
    }
    var manager = getManager(context, index);
    manager.manage(theme);
    if (context.registry) {
      context.registry.add(sheet);
    }
  };
  var unmanageSheet = function unmanageSheet(options) {
    if (!options.sheet) {
      return;
    }
    var manager = getManager(options.context, options.index);
    manager.unmanage(options.theme);
  };

  // A symbol replacement.
  var now = Date.now();
  var fnValuesNs = "fnValues" + now;
  var fnRuleNs = "fnStyle" + ++now;
  var functionPlugin = function functionPlugin() {
    return {
      onCreateRule: function onCreateRule(name, decl, options) {
        if (typeof decl !== 'function') return null;
        var rule = jss.createRule(name, {}, options);
        rule[fnRuleNs] = decl;
        return rule;
      },
      onProcessStyle: function onProcessStyle(style, rule) {
        // We need to extract function values from the declaration, so that we can keep core unaware of them.
        // We need to do that only once.
        // We don't need to extract functions on each style update, since this can happen only once.
        // We don't support function values inside of function rules.
        if (fnValuesNs in rule || fnRuleNs in rule) return style;
        var fnValues = {};
        for (var prop in style) {
          var value = style[prop];
          if (typeof value !== 'function') continue;
          delete style[prop];
          fnValues[prop] = value;
        }
        rule[fnValuesNs] = fnValues;
        return style;
      },
      onUpdate: function onUpdate(data, rule, sheet, options) {
        var styleRule = rule;
        var fnRule = styleRule[fnRuleNs];

        // If we have a style function, the entire rule is dynamic and style object
        // will be returned from that function.
        if (fnRule) {
          // Empty object will remove all currently defined props
          // in case function rule returns a falsy value.
          styleRule.style = fnRule(data) || {};
          {
            for (var prop in styleRule.style) {
              if (typeof styleRule.style[prop] === 'function') {
                warning(false, '[JSS] Function values inside function rules are not supported.');
                break;
              }
            }
          }
        }
        var fnValues = styleRule[fnValuesNs];

        // If we have a fn values map, it is a rule with function values.
        if (fnValues) {
          for (var _prop in fnValues) {
            styleRule.prop(_prop, fnValues[_prop](data), options);
          }
        }
      }
    };
  };

  var isObservable = function isObservable(value) {
    return value && value[$$observable] && value === value[$$observable]();
  };
  var observablePlugin = function observablePlugin(updateOptions) {
    return {
      onCreateRule: function onCreateRule(name, decl, options) {
        if (!isObservable(decl)) return null;
        var style$ = decl;
        var rule = jss.createRule(name, {}, options);

        // TODO
        // Call `stream.subscribe()` returns a subscription, which should be explicitly
        // unsubscribed from when we know this sheet is no longer needed.
        style$.subscribe(function (style) {
          for (var prop in style) {
            rule.prop(prop, style[prop], updateOptions);
          }
        });
        return rule;
      },
      onProcessRule: function onProcessRule(rule) {
        if (rule && rule.type !== 'style') return;
        var styleRule = rule;
        var style = styleRule.style;
        var _loop = function _loop(prop) {
          var value = style[prop];
          if (!isObservable(value)) return "continue";
          delete style[prop];
          value.subscribe({
            next: function next(nextValue) {
              styleRule.prop(prop, nextValue, updateOptions);
            }
          });
        };
        for (var prop in style) {
          var _ret = _loop(prop);
          if (_ret === "continue") continue;
        }
      }
    };
  };

  var semiWithNl = /;\n/;

  /**
   * Naive CSS parser.
   * - Supports only rule body (no selectors)
   * - Requires semicolon and new line after the value (except of last line)
   * - No nested rules support
   */
  var parse = function parse(cssText) {
    var style = {};
    var split = cssText.split(semiWithNl);
    for (var i = 0; i < split.length; i++) {
      var decl = (split[i] || '').trim();
      if (!decl) continue;
      var colonIndex = decl.indexOf(':');
      if (colonIndex === -1) {
        warning(false, "[JSS] Malformed CSS string \"" + decl + "\"");
        continue;
      }
      var prop = decl.substr(0, colonIndex).trim();
      var value = decl.substr(colonIndex + 1).trim();
      style[prop] = value;
    }
    return style;
  };
  var onProcessRule = function onProcessRule(rule) {
    if (typeof rule.style === 'string') {
      rule.style = parse(rule.style);
    }
  };
  function templatePlugin() {
    return {
      onProcessRule: onProcessRule
    };
  }

  var at = '@global';
  var atPrefix = '@global ';
  var GlobalContainerRule = /*#__PURE__*/function () {
    function GlobalContainerRule(key, styles, options) {
      this.type = 'global';
      this.at = at;
      this.isProcessed = false;
      this.key = key;
      this.options = options;
      this.rules = new jss.RuleList(_extends({}, options, {
        parent: this
      }));
      for (var selector in styles) {
        this.rules.add(selector, styles[selector]);
      }
      this.rules.process();
    }

    /**
     * Get a rule.
     */
    var _proto = GlobalContainerRule.prototype;
    _proto.getRule = function getRule(name) {
      return this.rules.get(name);
    }

    /**
     * Create and register rule, run plugins.
     */;
    _proto.addRule = function addRule(name, style, options) {
      var rule = this.rules.add(name, style, options);
      if (rule) this.options.jss.plugins.onProcessRule(rule);
      return rule;
    }

    /**
     * Replace rule, run plugins.
     */;
    _proto.replaceRule = function replaceRule(name, style, options) {
      var newRule = this.rules.replace(name, style, options);
      if (newRule) this.options.jss.plugins.onProcessRule(newRule);
      return newRule;
    }

    /**
     * Get index of a rule.
     */;
    _proto.indexOf = function indexOf(rule) {
      return this.rules.indexOf(rule);
    }

    /**
     * Generates a CSS string.
     */;
    _proto.toString = function toString(options) {
      return this.rules.toString(options);
    };
    return GlobalContainerRule;
  }();
  var GlobalPrefixedRule = /*#__PURE__*/function () {
    function GlobalPrefixedRule(key, style, options) {
      this.type = 'global';
      this.at = at;
      this.isProcessed = false;
      this.key = key;
      this.options = options;
      var selector = key.substr(atPrefix.length);
      this.rule = options.jss.createRule(selector, style, _extends({}, options, {
        parent: this
      }));
    }
    var _proto2 = GlobalPrefixedRule.prototype;
    _proto2.toString = function toString(options) {
      return this.rule ? this.rule.toString(options) : '';
    };
    return GlobalPrefixedRule;
  }();
  var separatorRegExp$1 = /\s*,\s*/g;
  function addScope(selector, scope) {
    var parts = selector.split(separatorRegExp$1);
    var scoped = '';
    for (var i = 0; i < parts.length; i++) {
      scoped += scope + " " + parts[i].trim();
      if (parts[i + 1]) scoped += ', ';
    }
    return scoped;
  }
  function handleNestedGlobalContainerRule(rule, sheet) {
    var options = rule.options,
      style = rule.style;
    var rules = style ? style[at] : null;
    if (!rules) return;
    for (var name in rules) {
      sheet.addRule(name, rules[name], _extends({}, options, {
        selector: addScope(name, rule.selector)
      }));
    }
    delete style[at];
  }
  function handlePrefixedGlobalRule(rule, sheet) {
    var options = rule.options,
      style = rule.style;
    for (var prop in style) {
      if (prop[0] !== '@' || prop.substr(0, at.length) !== at) continue;
      var selector = addScope(prop.substr(at.length), rule.selector);
      sheet.addRule(selector, style[prop], _extends({}, options, {
        selector: selector
      }));
      delete style[prop];
    }
  }

  /**
   * Convert nested rules to separate, remove them from original styles.
   */
  function jssGlobal() {
    function onCreateRule(name, styles, options) {
      if (!name) return null;
      if (name === at) {
        return new GlobalContainerRule(name, styles, options);
      }
      if (name[0] === '@' && name.substr(0, atPrefix.length) === atPrefix) {
        return new GlobalPrefixedRule(name, styles, options);
      }
      var parent = options.parent;
      if (parent) {
        if (parent.type === 'global' || parent.options.parent && parent.options.parent.type === 'global') {
          options.scoped = false;
        }
      }
      if (!options.selector && options.scoped === false) {
        options.selector = name;
      }
      return null;
    }
    function onProcessRule(rule, sheet) {
      if (rule.type !== 'style' || !sheet) return;
      handleNestedGlobalContainerRule(rule, sheet);
      handlePrefixedGlobalRule(rule, sheet);
    }
    return {
      onCreateRule: onCreateRule,
      onProcessRule: onProcessRule
    };
  }

  var isObject = function isObject(obj) {
    return obj && typeof obj === 'object' && !Array.isArray(obj);
  };
  var valueNs = "extendCurrValue" + Date.now();
  function mergeExtend(style, rule, sheet, newStyle) {
    var extendType = typeof style.extend;
    // Extend using a rule name.
    if (extendType === 'string') {
      if (!sheet) return;
      var refRule = sheet.getRule(style.extend);
      if (!refRule) return;
      if (refRule === rule) {
        warning(false, "[JSS] A rule tries to extend itself \n" + rule.toString());
        return;
      }
      var parent = refRule.options.parent;
      if (parent) {
        var originalStyle = parent.rules.raw[style.extend];
        extend(originalStyle, rule, sheet, newStyle);
      }
      return;
    }

    // Extend using an array.
    if (Array.isArray(style.extend)) {
      for (var index = 0; index < style.extend.length; index++) {
        var singleExtend = style.extend[index];
        var singleStyle = typeof singleExtend === 'string' ? _extends({}, style, {
          extend: singleExtend
        }) : style.extend[index];
        extend(singleStyle, rule, sheet, newStyle);
      }
      return;
    }

    // Extend is a style object.
    for (var prop in style.extend) {
      if (prop === 'extend') {
        extend(style.extend.extend, rule, sheet, newStyle);
        continue;
      }
      if (isObject(style.extend[prop])) {
        if (!(prop in newStyle)) newStyle[prop] = {};
        extend(style.extend[prop], rule, sheet, newStyle[prop]);
        continue;
      }
      newStyle[prop] = style.extend[prop];
    }
  }
  function mergeRest(style, rule, sheet, newStyle) {
    // Copy base style.
    for (var prop in style) {
      if (prop === 'extend') continue;
      if (isObject(newStyle[prop]) && isObject(style[prop])) {
        extend(style[prop], rule, sheet, newStyle[prop]);
        continue;
      }
      if (isObject(style[prop])) {
        newStyle[prop] = extend(style[prop], rule, sheet);
        continue;
      }
      newStyle[prop] = style[prop];
    }
  }

  /**
   * Recursively extend styles.
   */
  function extend(style, rule, sheet, newStyle) {
    if (newStyle === void 0) {
      newStyle = {};
    }
    mergeExtend(style, rule, sheet, newStyle);
    mergeRest(style, rule, sheet, newStyle);
    return newStyle;
  }

  /**
   * Handle `extend` property.
   */
  function jssExtend() {
    function onProcessStyle(style, rule, sheet) {
      if ('extend' in style) return extend(style, rule, sheet);
      return style;
    }
    function onChangeValue(value, prop, rule) {
      if (prop !== 'extend') return value;

      // Value is empty, remove properties set previously.
      if (value == null || value === false) {
        for (var key in rule[valueNs]) {
          rule.prop(key, null);
        }
        rule[valueNs] = null;
        return null;
      }
      if (typeof value === 'object') {
        for (var _key in value) {
          rule.prop(_key, value[_key]);
        }
        rule[valueNs] = value;
      }

      // Make sure we don't set the value in the core.
      return null;
    }
    return {
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  var separatorRegExp = /\s*,\s*/g;
  var parentRegExp = /&/g;
  var refRegExp = /\$([\w-]+)/g;

  /**
   * Convert nested rules to separate, remove them from original styles.
   */
  function jssNested() {
    // Get a function to be used for $ref replacement.
    function getReplaceRef(container, sheet) {
      return function (match, key) {
        var rule = container.getRule(key) || sheet && sheet.getRule(key);
        if (rule) {
          return rule.selector;
        }
        warning(false, "[JSS] Could not find the referenced rule \"" + key + "\" in \"" + (container.options.meta || container.toString()) + "\".");
        return key;
      };
    }
    function replaceParentRefs(nestedProp, parentProp) {
      var parentSelectors = parentProp.split(separatorRegExp);
      var nestedSelectors = nestedProp.split(separatorRegExp);
      var result = '';
      for (var i = 0; i < parentSelectors.length; i++) {
        var parent = parentSelectors[i];
        for (var j = 0; j < nestedSelectors.length; j++) {
          var nested = nestedSelectors[j];
          if (result) result += ', ';
          // Replace all & by the parent or prefix & with the parent.
          result += nested.indexOf('&') !== -1 ? nested.replace(parentRegExp, parent) : parent + " " + nested;
        }
      }
      return result;
    }
    function getOptions(rule, container, prevOptions) {
      // Options has been already created, now we only increase index.
      if (prevOptions) return _extends({}, prevOptions, {
        index: prevOptions.index + 1
      });
      var nestingLevel = rule.options.nestingLevel;
      nestingLevel = nestingLevel === undefined ? 1 : nestingLevel + 1;
      var options = _extends({}, rule.options, {
        nestingLevel: nestingLevel,
        index: container.indexOf(rule) + 1
      });
      // We don't need the parent name to be set options for chlid.
      delete options.name;
      return options;
    }
    function onProcessStyle(style, rule, sheet) {
      if (rule.type !== 'style') return style;
      var styleRule = rule;
      var container = styleRule.options.parent;
      var options;
      var replaceRef;
      for (var prop in style) {
        var isNested = prop.indexOf('&') !== -1;
        var isNestedConditional = prop[0] === '@';
        if (!isNested && !isNestedConditional) continue;
        options = getOptions(styleRule, container, options);
        if (isNested) {
          var selector = replaceParentRefs(prop, styleRule.selector);
          // Lazily create the ref replacer function just once for
          // all nested rules within the sheet.
          if (!replaceRef) replaceRef = getReplaceRef(container, sheet);
          // Replace all $refs.
          selector = selector.replace(refRegExp, replaceRef);
          var name = styleRule.key + "-" + prop;
          if ('replaceRule' in container) {
            // for backward compatibility
            container.replaceRule(name, style[prop], _extends({}, options, {
              selector: selector
            }));
          } else {
            container.addRule(name, style[prop], _extends({}, options, {
              selector: selector
            }));
          }
        } else if (isNestedConditional) {
          // Place conditional right after the parent rule to ensure right ordering.
          container.addRule(prop, {}, options).addRule(styleRule.key, style[prop], {
            selector: styleRule.selector
          });
        }
        delete style[prop];
      }
      return style;
    }
    return {
      onProcessStyle: onProcessStyle
    };
  }

  /**
   * Set selector.
   *
   * @param original rule
   * @param className class string
   * @return flag indicating function was successfull or not
   */
  function registerClass(rule, className) {
    // Skip falsy values
    if (!className) return true;

    // Support array of class names `{composes: ['foo', 'bar']}`
    if (Array.isArray(className)) {
      for (var index = 0; index < className.length; index++) {
        var isSetted = registerClass(rule, className[index]);
        if (!isSetted) return false;
      }
      return true;
    }

    // Support space separated class names `{composes: 'foo bar'}`
    if (className.indexOf(' ') > -1) {
      return registerClass(rule, className.split(' '));
    }
    var parent = rule.options.parent;

    // It is a ref to a local rule.
    if (className[0] === '$') {
      var refRule = parent.getRule(className.substr(1));
      if (!refRule) {
        warning(false, "[JSS] Referenced rule is not defined. \n" + rule.toString());
        return false;
      }
      if (refRule === rule) {
        warning(false, "[JSS] Cyclic composition detected. \n" + rule.toString());
        return false;
      }
      parent.classes[rule.key] += " " + parent.classes[refRule.key];
      return true;
    }
    parent.classes[rule.key] += " " + className;
    return true;
  }

  /**
   * Convert compose property to additional class, remove property from original styles.
   */
  function jssCompose() {
    function onProcessStyle(style, rule) {
      if (!('composes' in style)) return style;
      registerClass(rule, style.composes);
      // Remove composes property to prevent infinite loop.
      delete style.composes;
      return style;
    }
    return {
      onProcessStyle: onProcessStyle
    };
  }

  var uppercasePattern = /[A-Z]/g;
  var msPattern = /^ms-/;
  var cache = {};
  function toHyphenLower(match) {
    return '-' + match.toLowerCase();
  }
  function hyphenate(name) {
    if (cache.hasOwnProperty(name)) {
      return cache[name];
    }
    var hName = name.replace(uppercasePattern, toHyphenLower);
    return cache[name] = msPattern.test(hName) ? '-' + hName : hName;
  }

  /**
   * Convert camel cased property names to dash separated.
   */
  function convertCase(style) {
    var converted = {};
    for (var prop in style) {
      var key = prop.indexOf('--') === 0 ? prop : hyphenate(prop);
      converted[key] = style[prop];
    }
    if (style.fallbacks) {
      if (Array.isArray(style.fallbacks)) converted.fallbacks = style.fallbacks.map(convertCase);else converted.fallbacks = convertCase(style.fallbacks);
    }
    return converted;
  }

  /**
   * Allow camel cased property names by converting them back to dasherized.
   */
  function camelCase() {
    function onProcessStyle(style) {
      if (Array.isArray(style)) {
        // Handle rules like @font-face, which can have multiple styles in an array
        for (var index = 0; index < style.length; index++) {
          style[index] = convertCase(style[index]);
        }
        return style;
      }
      return convertCase(style);
    }
    function onChangeValue(value, prop, rule) {
      if (prop.indexOf('--') === 0) {
        return value;
      }
      var hyphenatedProp = hyphenate(prop);

      // There was no camel case in place
      if (prop === hyphenatedProp) return value;
      rule.prop(hyphenatedProp, value);

      // Core will ignore that property value we set the proper one above.
      return null;
    }
    return {
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  var px = jss.hasCSSTOMSupport && CSS ? CSS.px : 'px';
  var ms = jss.hasCSSTOMSupport && CSS ? CSS.ms : 'ms';
  var percent = jss.hasCSSTOMSupport && CSS ? CSS.percent : '%';

  /**
   * Generated jss-plugin-default-unit CSS property units
   */
  var defaultUnits = {
    // Animation properties
    'animation-delay': ms,
    'animation-duration': ms,
    // Background properties
    'background-position': px,
    'background-position-x': px,
    'background-position-y': px,
    'background-size': px,
    // Border Properties
    border: px,
    'border-bottom': px,
    'border-bottom-left-radius': px,
    'border-bottom-right-radius': px,
    'border-bottom-width': px,
    'border-left': px,
    'border-left-width': px,
    'border-radius': px,
    'border-right': px,
    'border-right-width': px,
    'border-top': px,
    'border-top-left-radius': px,
    'border-top-right-radius': px,
    'border-top-width': px,
    'border-width': px,
    'border-block': px,
    'border-block-end': px,
    'border-block-end-width': px,
    'border-block-start': px,
    'border-block-start-width': px,
    'border-block-width': px,
    'border-inline': px,
    'border-inline-end': px,
    'border-inline-end-width': px,
    'border-inline-start': px,
    'border-inline-start-width': px,
    'border-inline-width': px,
    'border-start-start-radius': px,
    'border-start-end-radius': px,
    'border-end-start-radius': px,
    'border-end-end-radius': px,
    // Margin properties
    margin: px,
    'margin-bottom': px,
    'margin-left': px,
    'margin-right': px,
    'margin-top': px,
    'margin-block': px,
    'margin-block-end': px,
    'margin-block-start': px,
    'margin-inline': px,
    'margin-inline-end': px,
    'margin-inline-start': px,
    // Padding properties
    padding: px,
    'padding-bottom': px,
    'padding-left': px,
    'padding-right': px,
    'padding-top': px,
    'padding-block': px,
    'padding-block-end': px,
    'padding-block-start': px,
    'padding-inline': px,
    'padding-inline-end': px,
    'padding-inline-start': px,
    // Mask properties
    'mask-position-x': px,
    'mask-position-y': px,
    'mask-size': px,
    // Width and height properties
    height: px,
    width: px,
    'min-height': px,
    'max-height': px,
    'min-width': px,
    'max-width': px,
    // Position properties
    bottom: px,
    left: px,
    top: px,
    right: px,
    inset: px,
    'inset-block': px,
    'inset-block-end': px,
    'inset-block-start': px,
    'inset-inline': px,
    'inset-inline-end': px,
    'inset-inline-start': px,
    // Shadow properties
    'box-shadow': px,
    'text-shadow': px,
    // Column properties
    'column-gap': px,
    'column-rule': px,
    'column-rule-width': px,
    'column-width': px,
    // Font and text properties
    'font-size': px,
    'font-size-delta': px,
    'letter-spacing': px,
    'text-decoration-thickness': px,
    'text-indent': px,
    'text-stroke': px,
    'text-stroke-width': px,
    'word-spacing': px,
    // Motion properties
    motion: px,
    'motion-offset': px,
    // Outline properties
    outline: px,
    'outline-offset': px,
    'outline-width': px,
    // Perspective properties
    perspective: px,
    'perspective-origin-x': percent,
    'perspective-origin-y': percent,
    // Transform properties
    'transform-origin': percent,
    'transform-origin-x': percent,
    'transform-origin-y': percent,
    'transform-origin-z': percent,
    // Transition properties
    'transition-delay': ms,
    'transition-duration': ms,
    // Alignment properties
    'vertical-align': px,
    'flex-basis': px,
    // Some random properties
    'shape-margin': px,
    size: px,
    gap: px,
    // Grid properties
    grid: px,
    'grid-gap': px,
    'row-gap': px,
    'grid-row-gap': px,
    'grid-column-gap': px,
    'grid-template-rows': px,
    'grid-template-columns': px,
    'grid-auto-rows': px,
    'grid-auto-columns': px,
    // Not existing properties.
    // Used to avoid issues with jss-plugin-expand integration.
    'box-shadow-x': px,
    'box-shadow-y': px,
    'box-shadow-blur': px,
    'box-shadow-spread': px,
    'font-line-height': px,
    'text-shadow-x': px,
    'text-shadow-y': px,
    'text-shadow-blur': px
  };

  /**
   * Clones the object and adds a camel cased property version.
   */
  function addCamelCasedVersion(obj) {
    var regExp = /(-[a-z])/g;
    var replace = function replace(str) {
      return str[1].toUpperCase();
    };
    var newObj = {};
    for (var key in obj) {
      newObj[key] = obj[key];
      newObj[key.replace(regExp, replace)] = obj[key];
    }
    return newObj;
  }
  var units = addCamelCasedVersion(defaultUnits);

  /**
   * Recursive deep style passing function
   */
  function iterate(prop, value, options) {
    if (value == null) return value;
    if (Array.isArray(value)) {
      for (var i = 0; i < value.length; i++) {
        value[i] = iterate(prop, value[i], options);
      }
    } else if (typeof value === 'object') {
      if (prop === 'fallbacks') {
        for (var innerProp in value) {
          value[innerProp] = iterate(innerProp, value[innerProp], options);
        }
      } else {
        for (var _innerProp in value) {
          value[_innerProp] = iterate(prop + "-" + _innerProp, value[_innerProp], options);
        }
      }
      // eslint-disable-next-line no-restricted-globals
    } else if (typeof value === 'number' && isNaN(value) === false) {
      var unit = options[prop] || units[prop];

      // Add the unit if available, except for the special case of 0px.
      if (unit && !(value === 0 && unit === px)) {
        return typeof unit === 'function' ? unit(value).toString() : "" + value + unit;
      }
      return value.toString();
    }
    return value;
  }

  /**
   * Add unit to numeric values.
   */
  function defaultUnit(options) {
    if (options === void 0) {
      options = {};
    }
    var camelCasedOptions = addCamelCasedVersion(options);
    function onProcessStyle(style, rule) {
      if (rule.type !== 'style') return style;
      for (var prop in style) {
        style[prop] = iterate(prop, style[prop], camelCasedOptions);
      }
      return style;
    }
    function onChangeValue(value, prop) {
      return iterate(prop, value, camelCasedOptions);
    }
    return {
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  /**
   * A scheme for converting properties from array to regular style.
   * All properties listed below will be transformed to a string separated by space.
   */
  var propArray = {
    'background-size': true,
    'background-position': true,
    border: true,
    'border-bottom': true,
    'border-left': true,
    'border-top': true,
    'border-right': true,
    'border-radius': true,
    'border-image': true,
    'border-width': true,
    'border-style': true,
    'border-color': true,
    'box-shadow': true,
    flex: true,
    margin: true,
    padding: true,
    outline: true,
    'transform-origin': true,
    transform: true,
    transition: true
  };

  /**
   * A scheme for converting arrays to regular styles inside of objects.
   * For e.g.: "{position: [0, 0]}" => "background-position: 0 0;".
   */
  var propArrayInObj = {
    position: true,
    // background-position
    size: true // background-size
  };

  /**
   * A scheme for parsing and building correct styles from passed objects.
   */
  var propObj = {
    padding: {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
    },
    margin: {
      top: 0,
      right: 0,
      bottom: 0,
      left: 0
    },
    background: {
      attachment: null,
      color: null,
      image: null,
      position: null,
      repeat: null
    },
    border: {
      width: null,
      style: null,
      color: null
    },
    'border-top': {
      width: null,
      style: null,
      color: null
    },
    'border-right': {
      width: null,
      style: null,
      color: null
    },
    'border-bottom': {
      width: null,
      style: null,
      color: null
    },
    'border-left': {
      width: null,
      style: null,
      color: null
    },
    outline: {
      width: null,
      style: null,
      color: null
    },
    'list-style': {
      type: null,
      position: null,
      image: null
    },
    transition: {
      property: null,
      duration: null,
      'timing-function': null,
      timingFunction: null,
      // Needed for avoiding comilation issues with jss-plugin-camel-case
      delay: null
    },
    animation: {
      name: null,
      duration: null,
      'timing-function': null,
      timingFunction: null,
      // Needed to avoid compilation issues with jss-plugin-camel-case
      delay: null,
      'iteration-count': null,
      iterationCount: null,
      // Needed to avoid compilation issues with jss-plugin-camel-case
      direction: null,
      'fill-mode': null,
      fillMode: null,
      // Needed to avoid compilation issues with jss-plugin-camel-case
      'play-state': null,
      playState: null // Needed to avoid compilation issues with jss-plugin-camel-case
    },

    'box-shadow': {
      x: 0,
      y: 0,
      blur: 0,
      spread: 0,
      color: null,
      inset: null
    },
    'text-shadow': {
      x: 0,
      y: 0,
      blur: null,
      color: null
    }
  };

  /**
   * A scheme for converting non-standart properties inside object.
   * For e.g.: include 'border-radius' property inside 'border' object.
   */
  var customPropObj = {
    border: {
      radius: 'border-radius',
      image: 'border-image',
      width: 'border-width',
      style: 'border-style',
      color: 'border-color'
    },
    'border-bottom': {
      width: 'border-bottom-width',
      style: 'border-bottom-style',
      color: 'border-bottom-color'
    },
    'border-top': {
      width: 'border-top-width',
      style: 'border-top-style',
      color: 'border-top-color'
    },
    'border-left': {
      width: 'border-left-width',
      style: 'border-left-style',
      color: 'border-left-color'
    },
    'border-right': {
      width: 'border-right-width',
      style: 'border-right-style',
      color: 'border-right-color'
    },
    background: {
      size: 'background-size',
      image: 'background-image'
    },
    font: {
      style: 'font-style',
      variant: 'font-variant',
      weight: 'font-weight',
      stretch: 'font-stretch',
      size: 'font-size',
      family: 'font-family',
      lineHeight: 'line-height',
      // Needed to avoid compilation issues with jss-plugin-camel-case
      'line-height': 'line-height'
    },
    flex: {
      grow: 'flex-grow',
      basis: 'flex-basis',
      direction: 'flex-direction',
      wrap: 'flex-wrap',
      flow: 'flex-flow',
      shrink: 'flex-shrink'
    },
    align: {
      self: 'align-self',
      items: 'align-items',
      content: 'align-content'
    },
    grid: {
      'template-columns': 'grid-template-columns',
      templateColumns: 'grid-template-columns',
      'template-rows': 'grid-template-rows',
      templateRows: 'grid-template-rows',
      'template-areas': 'grid-template-areas',
      templateAreas: 'grid-template-areas',
      template: 'grid-template',
      'auto-columns': 'grid-auto-columns',
      autoColumns: 'grid-auto-columns',
      'auto-rows': 'grid-auto-rows',
      autoRows: 'grid-auto-rows',
      'auto-flow': 'grid-auto-flow',
      autoFlow: 'grid-auto-flow',
      row: 'grid-row',
      column: 'grid-column',
      'row-start': 'grid-row-start',
      rowStart: 'grid-row-start',
      'row-end': 'grid-row-end',
      rowEnd: 'grid-row-end',
      'column-start': 'grid-column-start',
      columnStart: 'grid-column-start',
      'column-end': 'grid-column-end',
      columnEnd: 'grid-column-end',
      area: 'grid-area',
      gap: 'grid-gap',
      'row-gap': 'grid-row-gap',
      rowGap: 'grid-row-gap',
      'column-gap': 'grid-column-gap',
      columnGap: 'grid-column-gap'
    }
  };

  /* eslint-disable no-use-before-define */

  /**
   * Map values by given prop.
   *
   * @param {Array} array of values
   * @param {String} original property
   * @param {String} original rule
   * @return {String} mapped values
   */
  function mapValuesByProp(value, prop, rule) {
    return value.map(function (item) {
      return objectToArray(item, prop, rule, false, true);
    });
  }

  /**
   * Convert array to nested array, if needed
   */
  function processArray(value, prop, scheme, rule) {
    if (scheme[prop] == null) return value;
    if (value.length === 0) return [];
    if (Array.isArray(value[0])) return processArray(value[0], prop, scheme, rule);
    if (typeof value[0] === 'object') {
      return mapValuesByProp(value, prop, rule);
    }
    return [value];
  }

  /**
   * Convert object to array.
   */
  function objectToArray(value, prop, rule, isFallback, isInArray) {
    if (!(propObj[prop] || customPropObj[prop])) return [];
    var result = [];

    // Check if exists any non-standard property
    if (customPropObj[prop]) {
      // eslint-disable-next-line no-param-reassign
      value = customPropsToStyle(value, rule, customPropObj[prop], isFallback);
    }

    // Pass throught all standart props
    if (Object.keys(value).length) {
      for (var baseProp in propObj[prop]) {
        if (value[baseProp]) {
          if (Array.isArray(value[baseProp])) {
            result.push(propArrayInObj[baseProp] === null ? value[baseProp] : value[baseProp].join(' '));
          } else result.push(value[baseProp]);
          continue;
        }

        // Add default value from props config.
        if (propObj[prop][baseProp] != null) {
          result.push(propObj[prop][baseProp]);
        }
      }
    }
    if (!result.length || isInArray) return result;
    return [result];
  }

  /**
   * Convert custom properties values to styles adding them to rule directly
   */
  function customPropsToStyle(value, rule, customProps, isFallback) {
    for (var prop in customProps) {
      var propName = customProps[prop];

      // If current property doesn't exist already in rule - add new one
      if (typeof value[prop] !== 'undefined' && (isFallback || !rule.prop(propName))) {
        var _styleDetector;
        var appendedValue = styleDetector((_styleDetector = {}, _styleDetector[propName] = value[prop], _styleDetector), rule)[propName];

        // Add style directly in rule
        if (isFallback) rule.style.fallbacks[propName] = appendedValue;else rule.style[propName] = appendedValue;
      }
      // Delete converted property to avoid double converting
      delete value[prop];
    }
    return value;
  }

  /**
   * Detect if a style needs to be converted.
   */
  function styleDetector(style, rule, isFallback) {
    for (var prop in style) {
      var value = style[prop];
      if (Array.isArray(value)) {
        // Check double arrays to avoid recursion.
        if (!Array.isArray(value[0])) {
          if (prop === 'fallbacks') {
            for (var index = 0; index < style.fallbacks.length; index++) {
              style.fallbacks[index] = styleDetector(style.fallbacks[index], rule, true);
            }
            continue;
          }
          style[prop] = processArray(value, prop, propArray, rule);
          // Avoid creating properties with empty values
          if (!style[prop].length) delete style[prop];
        }
      } else if (typeof value === 'object') {
        if (prop === 'fallbacks') {
          style.fallbacks = styleDetector(style.fallbacks, rule, true);
          continue;
        }
        style[prop] = objectToArray(value, prop, rule, isFallback);
        // Avoid creating properties with empty values
        if (!style[prop].length) delete style[prop];
      }

      // Maybe a computed value resulting in an empty string
      else if (style[prop] === '') delete style[prop];
    }
    return style;
  }

  /**
   * Adds possibility to write expanded styles.
   */
  function jssExpand() {
    function onProcessStyle(style, rule) {
      if (!style || rule.type !== 'style') return style;
      if (Array.isArray(style)) {
        // Pass rules one by one and reformat them
        for (var index = 0; index < style.length; index++) {
          style[index] = styleDetector(style[index], rule);
        }
        return style;
      }
      return styleDetector(style, rule);
    }
    return {
      onProcessStyle: onProcessStyle
    };
  }

  /**
   * Add vendor prefix to a property name when needed.
   */
  function jssVendorPrefixer() {
    function onProcessRule(rule) {
      if (rule.type === 'keyframes') {
        var atRule = rule;
        atRule.at = vendor__namespace.supportedKeyframes(atRule.at);
      }
    }
    function prefixStyle(style) {
      for (var prop in style) {
        var value = style[prop];
        if (prop === 'fallbacks' && Array.isArray(value)) {
          style[prop] = value.map(prefixStyle);
          continue;
        }
        var changeProp = false;
        var supportedProp = vendor__namespace.supportedProperty(prop);
        if (supportedProp && supportedProp !== prop) changeProp = true;
        var changeValue = false;
        var supportedValue = vendor__namespace.supportedValue(supportedProp, jss.toCssValue(value));
        if (supportedValue && supportedValue !== value) changeValue = true;
        if (changeProp || changeValue) {
          if (changeProp) delete style[prop];
          style[supportedProp || prop] = supportedValue || value;
        }
      }
      return style;
    }
    function onProcessStyle(style, rule) {
      if (rule.type !== 'style') return style;
      return prefixStyle(style);
    }
    function onChangeValue(value, prop) {
      return vendor__namespace.supportedValue(prop, jss.toCssValue(value)) || value;
    }
    return {
      onProcessRule: onProcessRule,
      onProcessStyle: onProcessStyle,
      onChangeValue: onChangeValue
    };
  }

  /**
   * Sort props by length.
   */
  function jssPropsSort() {
    var sort = function sort(prop0, prop1) {
      if (prop0.length === prop1.length) {
        return prop0 > prop1 ? 1 : -1;
      }
      return prop0.length - prop1.length;
    };
    return {
      onProcessStyle: function onProcessStyle(style, rule) {
        if (rule.type !== 'style') return style;
        var newStyle = {};
        var props = Object.keys(style).sort(sort);
        for (var i = 0; i < props.length; i++) {
          newStyle[props[i]] = style[props[i]];
        }
        return newStyle;
      }
    };
  }

  var create$1 = function create(options) {
    if (options === void 0) {
      options = {};
    }
    return {
      plugins: [functionPlugin(), observablePlugin(options.observable), templatePlugin(), jssGlobal(), jssExtend(), jssNested(), jssCompose(), camelCase(), defaultUnit(options.defaultUnit), jssExpand(), jssVendorPrefixer(), jssPropsSort()]
    };
  };

  var defaultJss$1 = jss.create(create$1());

  var sheetsMeta = new WeakMap();
  var getMeta = function getMeta(sheet) {
    return sheetsMeta.get(sheet);
  };
  var addMeta = function addMeta(sheet, meta) {
    sheetsMeta.set(sheet, meta);
  };

  var getStyles = function getStyles(options) {
    var styles = options.styles;
    if (typeof styles !== 'function') {
      return styles;
    }
    warning(styles.length !== 0, "[JSS] <" + (options.name || 'Hook') + " />'s styles function doesn't rely on the \"theme\" argument. We recommend declaring styles as an object instead.");
    return styles(options.theme);
  };
  function getSheetOptions(options, link) {
    var minify;
    if (options.context.id && options.context.id.minify != null) {
      minify = options.context.id.minify;
    }
    var classNamePrefix = options.context.classNamePrefix || '';
    if (options.name && !minify) {
      classNamePrefix += options.name.replace(/\s/g, '-') + "-";
    }
    var meta = '';
    if (options.name) meta = options.name + ", ";
    meta += typeof options.styles === 'function' ? 'Themed' : 'Unthemed';
    return _extends({}, options.sheetOptions, {
      index: options.index,
      meta: meta,
      classNamePrefix: classNamePrefix,
      link: link,
      generateId: options.sheetOptions && options.sheetOptions.generateId ? options.sheetOptions.generateId : options.context.generateId
    });
  }
  var createStyleSheet = function createStyleSheet(options) {
    if (options.context.disableStylesGeneration) {
      return undefined;
    }
    var manager = getManager(options.context, options.index);
    var existingSheet = manager.get(options.theme);
    if (existingSheet) {
      return existingSheet;
    }
    var jss$1 = options.context.jss || defaultJss$1;
    var styles = getStyles(options);
    var dynamicStyles = jss.getDynamicStyles(styles);
    var sheet = jss$1.createStyleSheet(styles, getSheetOptions(options, dynamicStyles !== null));
    addMeta(sheet, {
      dynamicStyles: dynamicStyles,
      styles: styles
    });
    manager.add(options.theme, sheet);
    return sheet;
  };
  var removeDynamicRules = function removeDynamicRules(sheet, rules) {
    // Loop over each dynamic rule and remove the dynamic rule
    // We can't just remove the whole sheet as this has all of the rules for every component instance
    for (var key in rules) {
      sheet.deleteRule(rules[key]);
    }
  };
  var updateDynamicRules = function updateDynamicRules(data, sheet, rules) {
    // Loop over each dynamic rule and update it
    // We can't just update the whole sheet as this has all of the rules for every component instance
    for (var key in rules) {
      sheet.updateOne(rules[key], data);
    }
  };
  var addDynamicRules = function addDynamicRules(sheet, data) {
    var meta = getMeta(sheet);
    if (!meta) {
      return undefined;
    }
    var rules = {};

    // Loop over each dynamic rule and add it to the stylesheet
    for (var key in meta.dynamicStyles) {
      var initialRuleCount = sheet.rules.index.length;
      var originalRule = sheet.addRule(key, meta.dynamicStyles[key]);

      // Loop through all created rules, fixes updating dynamic rules
      for (var i = initialRuleCount; i < sheet.rules.index.length; i++) {
        var rule = sheet.rules.index[i];
        sheet.updateOne(rule, data);

        // If it's the original rule, we need to add it by the correct key so the hook and hoc
        // can correctly concat the dynamic class with the static one
        rules[originalRule === rule ? key : rule.key] = rule;
      }
    }
    return rules;
  };

  var getSheetClasses = function getSheetClasses(sheet, dynamicRules) {
    if (!dynamicRules) {
      return sheet.classes;
    }
    var meta = getMeta(sheet);
    if (!meta) {
      return sheet.classes;
    }
    var classes = {};
    for (var key in meta.styles) {
      classes[key] = sheet.classes[key];
      if (key in dynamicRules) {
        classes[key] += " " + sheet.classes[dynamicRules[key].key];
      }
    }
    return classes;
  };
  var getSheetClasses$1 = getSheetClasses;

  var _excluded$2 = ["index", "theming", "name"];
  function getUseInsertionEffect(isSSR) {
    return isSSR ? React.useEffect : React.useInsertionEffect ||
    // React 18+ (https://github.com/reactwg/react-18/discussions/110)
    React.useLayoutEffect;
  }
  var noTheme = {};
  var createUseStyles = function createUseStyles(styles, options) {
    if (options === void 0) {
      options = {};
    }
    var _options = options,
      _options$index = _options.index,
      index = _options$index === void 0 ? getSheetIndex$1() : _options$index,
      theming$1 = _options.theming,
      name = _options.name,
      sheetOptions = _objectWithoutPropertiesLoose(_options, _excluded$2);
    var ThemeContext = theming$1 && theming$1.context || theming.ThemeContext;
    var useTheme = function useTheme(theme) {
      if (typeof styles === 'function') {
        return theme || React.useContext(ThemeContext) || noTheme;
      }
      return noTheme;
    };
    var emptyObject = {};
    return function useStyles(data) {
      var isFirstMount = React.useRef(true);
      var context = React.useContext(JssContext);
      var theme = useTheme(data && data.theme);
      var _useMemo = React.useMemo(function () {
          var newSheet = createStyleSheet({
            context: context,
            styles: styles,
            name: name,
            theme: theme,
            index: index,
            sheetOptions: sheetOptions
          });
          if (newSheet && context.isSSR) {
            // manage immediately during SSRs. browsers will manage the sheet through useInsertionEffect below
            manageSheet({
              index: index,
              context: context,
              sheet: newSheet,
              theme: theme
            });
          }
          return [newSheet, newSheet ? addDynamicRules(newSheet, data) : null];
        }, [context, theme]),
        sheet = _useMemo[0],
        dynamicRules = _useMemo[1];
      getUseInsertionEffect(context.isSSR)(function () {
        // We only need to update the rules on a subsequent update and not in the first mount
        if (sheet && dynamicRules && !isFirstMount.current) {
          updateDynamicRules(data, sheet, dynamicRules);
        }
      }, [data]);
      getUseInsertionEffect(context.isSSR)(function () {
        if (sheet) {
          manageSheet({
            index: index,
            context: context,
            sheet: sheet,
            theme: theme
          });
        }
        return function () {
          if (sheet) {
            unmanageSheet({
              index: index,
              context: context,
              sheet: sheet,
              theme: theme
            });

            // when sheet changes, remove related dynamic rules
            if (dynamicRules) {
              removeDynamicRules(sheet, dynamicRules);
            }
          }
        };
      }, [sheet]);
      var classes = React.useMemo(function () {
        return sheet && dynamicRules ? getSheetClasses$1(sheet, dynamicRules) : emptyObject;
      }, [sheet, dynamicRules]);
      React.useDebugValue(classes);
      React.useDebugValue(theme === noTheme ? 'No theme' : theme);
      React.useEffect(function () {
        isFirstMount.current = false;
      });
      return classes;
    };
  };
  var createUseStyles$1 = createUseStyles;

  var _excluded$1 = ["index", "theming", "injectTheme"];
  var NoRenderer = function NoRenderer(props) {
    return props.children || null;
  };

  /**
   * HOC creator function that wrapps the user component.
   *
   * `withStyles(styles, [options])(Component)`
   */

  var createWithStyles = function createWithStyles(styles, options) {
    if (options === void 0) {
      options = {};
    }
    var _options = options,
      _options$index = _options.index,
      index = _options$index === void 0 ? getSheetIndex$1() : _options$index,
      theming$1 = _options.theming,
      injectTheme = _options.injectTheme,
      sheetOptions = _objectWithoutPropertiesLoose(_options, _excluded$1);
    var ThemeContext = theming$1 ? theming$1.context : theming.ThemeContext;
    return function (InnerComponent) {
      if (InnerComponent === void 0) {
        InnerComponent = NoRenderer;
      }
      var displayName = getDisplayName$1(InnerComponent);
      var mergeClassesProp = memoize$1(function (sheetClasses, classesProp) {
        return classesProp ? mergeClasses$1(sheetClasses, classesProp) : sheetClasses;
      });
      var hookOptions = Object.assign(sheetOptions, {
        theming: theming$1,
        index: index,
        name: displayName
      });
      var useStyles = createUseStyles$1(styles, hookOptions);
      var WithStyles = /*#__PURE__*/React__namespace.forwardRef(function (props, ref) {
        var theme = React__namespace.useContext(ThemeContext);
        var newProps = _extends({}, props);
        if (injectTheme && newProps.theme == null) {
          newProps.theme = theme;
        }
        var sheetClasses = useStyles(newProps);
        var classes = mergeClassesProp(sheetClasses, props.classes);
        return /*#__PURE__*/React__namespace.createElement(InnerComponent, _extends({}, newProps, {
          classes: classes,
          ref: ref
        }));
      });
      WithStyles.displayName = "WithStyles(" + displayName + ")";
      WithStyles.defaultProps = _extends({}, InnerComponent.defaultProps);
      WithStyles.InnerComponent = InnerComponent;
      return hoistNonReactStatics(WithStyles, InnerComponent);
    };
  };
  var withStyles = createWithStyles;

  var initialContext = {};
  function JssProvider(props) {
    var managersRef = React__namespace.useRef({});
    var prevContextRef = React__namespace.useRef();
    var registryRef = React__namespace.useRef(null);
    var createContext = function createContext(parentContext, prevContext) {
      if (prevContext === void 0) {
        prevContext = initialContext;
      }
      var registry = props.registry,
        classNamePrefix = props.classNamePrefix,
        jss$1 = props.jss,
        generateId = props.generateId,
        disableStylesGeneration = props.disableStylesGeneration,
        media = props.media,
        id = props.id,
        isSSR = props.isSSR;
      var context = _extends({}, parentContext);
      if (registry) {
        context.registry = registry;

        // This way we identify a new request on the server, because user will create
        // a new Registry instance for each.
        if (registry !== registryRef.current) {
          // We reset managers because we have to regenerate all sheets for the new request.
          managersRef.current = {};
          registryRef.current = registry;
        }
      }
      context.managers = managersRef.current;
      if (id !== undefined) {
        context.id = id;
      }
      if (generateId !== undefined) {
        context.generateId = generateId;
      } else if (!context.generateId || !prevContext || context.id !== prevContext.id) {
        context.generateId = jss.createGenerateId(context.id);
      }
      if (classNamePrefix) {
        context.classNamePrefix = (context.classNamePrefix || '') + classNamePrefix;
      }
      if (media !== undefined) {
        context.media = media;
      }
      if (jss$1) {
        context.jss = jss$1;
      }
      if (disableStylesGeneration !== undefined) {
        context.disableStylesGeneration = disableStylesGeneration;
      }
      if (isSSR !== undefined) {
        context.isSSR = isSSR;
      }
      if (prevContext && shallowEqual.shallowEqualObjects(prevContext, context)) {
        return prevContext;
      }
      return context;
    };
    var renderProvider = function renderProvider(parentContext) {
      var children = props.children;
      var context = createContext(parentContext, prevContextRef.current);
      prevContextRef.current = context;
      return /*#__PURE__*/React__namespace.createElement(JssContext.Provider, {
        value: context
      }, children);
    };
    return /*#__PURE__*/React__namespace.createElement(JssContext.Consumer, null, renderProvider);
  }

  var _excluded = ["shouldForwardProp"];

  // eslint-disable-next-line no-unused-vars
  var parseStyles = function parseStyles(args) {
    var dynamicStyles = [];
    var staticStyle;
    var labels = [];

    // Not using ...rest to optimize perf.
    for (var key in args) {
      var style = args[key];
      if (!style) continue;
      if (typeof style === 'function') {
        dynamicStyles.push(style);
      } else {
        if (!staticStyle) staticStyle = {};
        Object.assign(staticStyle, style);
        var _staticStyle = staticStyle,
          _label = _staticStyle.label;
        if (_label) {
          if (labels.indexOf(_label) === -1) labels.push(_label);
        }
      }
    }
    var styles = {};
    var label = labels.length === 0 ? 'sc' : labels.join('-');
    if (staticStyle) {
      // Label should not leak to the core.
      if ('label' in staticStyle) delete staticStyle.label;
      styles[label] = staticStyle;
    }

    // When there is only one function rule, we don't need to wrap it.
    if (dynamicStyles.length === 1) {
      styles.scd = dynamicStyles[0];
    }

    // We create a new function rule which will call all other function rules
    // and merge the styles they return.
    if (dynamicStyles.length > 1) {
      styles.scd = function (props) {
        var merged = {};
        for (var i = 0; i < dynamicStyles.length; i++) {
          var dynamicStyle = dynamicStyles[i](props);
          if (dynamicStyle) Object.assign(merged, dynamicStyle);
        }
        return merged;
      };
    }
    return {
      styles: styles,
      label: label
    };
  };
  var shouldForwardPropSymbol = Symbol('react-jss-styled');
  var getShouldForwardProp = function getShouldForwardProp(tagOrComponent, options) {
    var shouldForwardProp = options.shouldForwardProp;
    var childShouldForwardProp = tagOrComponent[shouldForwardPropSymbol];
    var finalShouldForwardProp = shouldForwardProp || childShouldForwardProp;
    if (shouldForwardProp && childShouldForwardProp) {
      finalShouldForwardProp = function finalShouldForwardProp(prop) {
        return childShouldForwardProp(prop) && shouldForwardProp(prop);
      };
    }
    return finalShouldForwardProp;
  };
  var getChildProps = function getChildProps(props, shouldForwardProp, isTag) {
    var childProps = {};
    for (var prop in props) {
      if (shouldForwardProp) {
        if (shouldForwardProp(prop) === true) {
          childProps[prop] = props[prop];
        }
        continue;
      }

      // We don't want to pass non-dom props to the DOM.
      if (isTag) {
        if (isPropValid(prop)) {
          childProps[prop] = props[prop];
        }
        continue;
      }
      childProps[prop] = props[prop];
    }
    return childProps;
  };

  // eslint-disable-next-line no-unused-vars
  var configureStyled = function configureStyled(tagOrComponent, options) {
    if (options === void 0) {
      options = {};
    }
    var _options = options,
      theming$1 = _options.theming;
    var isTag = typeof tagOrComponent === 'string';
    var ThemeContext = theming$1 ? theming$1.context : theming.ThemeContext;
    var shouldForwardProp = getShouldForwardProp(tagOrComponent, options);
    var _options2 = options;
      _options2.shouldForwardProp;
      var hookOptions = _objectWithoutPropertiesLoose(_options2, _excluded);
    return function createStyledComponent() {
      // eslint-disable-next-line prefer-rest-params
      var _parseStyles = parseStyles(arguments),
        styles = _parseStyles.styles,
        label = _parseStyles.label;
      var useStyles = createUseStyles$1(styles, hookOptions);
      var Styled = function Styled(props) {
        var as = props.as,
          className = props.className;
        var theme = React__namespace.useContext(ThemeContext);
        var propsWithTheme = Object.assign({
          theme: theme
        }, props);
        var classes = useStyles(propsWithTheme);
        var childProps = getChildProps(props, shouldForwardProp, isTag);
        var classNames = ((classes[label] || classes.sc || '') + " " + (classes.scd || '')).trim();
        childProps.className = className ? className + " " + classNames : classNames;
        if (!isTag && shouldForwardProp) {
          tagOrComponent[shouldForwardPropSymbol] = shouldForwardProp;
        }
        if (isTag && as) {
          return /*#__PURE__*/React__namespace.createElement(as, childProps);
        }
        return /*#__PURE__*/React__namespace.createElement(tagOrComponent, childProps);
      };
      return Styled;
    };
  };
  var configureStyled$1 = configureStyled;

  // I have been trying to benchmark and I have seen a slow down after about 10k rules.
  // Since we are in a single sheet mode, user shouldn't care about this.
  var MAX_RULES_PER_SHEET = 10000;
  var defaultJss = jss.create(create$1());
  var createCss = function createCss(jss) {
    if (jss === void 0) {
      jss = defaultJss;
    }
    var cache = new Map();
    var ruleIndex = 0;
    var sheet;
    var getSheet = function getSheet() {
      if (!sheet || sheet.rules.index.length > MAX_RULES_PER_SHEET) {
        sheet = jss.createStyleSheet().attach();
      }
      return sheet;
    };
    function css() {
      // eslint-disable-next-line prefer-rest-params
      var args = arguments;

      // We can avoid the need for stringification with a babel plugin,
      // which could generate a hash at build time and add it to the object.
      var argsStr = JSON.stringify(args);
      var cached = cache.get(argsStr);
      if (cached) return cached.className;
      var flatArgs = [];

      // Flatten arguments which can be
      // - style objects
      // - array of style objects
      // - arrays of style objects
      for (var argIndex in args) {
        var arg = args[argIndex];
        if (!Array.isArray(arg)) {
          flatArgs.push(arg);
          continue;
        }
        for (var innerArgIndex = 0; innerArgIndex < arg.length; innerArgIndex++) {
          flatArgs.push(arg[innerArgIndex]);
        }
      }
      var mergedStyle = {};
      var labels = [];
      for (var i = 0; i < flatArgs.length; i++) {
        var style = flatArgs[i];
        if (!style) continue;
        var styleObject = style;
        // It can be a class name that css() has previously generated.
        if (typeof style === 'string') {
          // eslint-disable-next-line no-shadow
          var _cached = cache.get(style);
          if (_cached) {
            // eslint-disable-next-line prefer-spread
            if (_cached.labels.length) labels.push.apply(labels, _cached.labels);
            styleObject = _cached.style;
          }
        }
        if (styleObject.label && labels.indexOf(styleObject.label) === -1) labels.push(styleObject.label);
        Object.assign(mergedStyle, styleObject);
      }
      delete mergedStyle.label;
      var label = labels.length === 0 ? 'css' : labels.join('-');
      var key = label + "-" + ruleIndex++;
      getSheet().addRule(key, mergedStyle);
      var className = getSheet().classes[key];
      var cacheValue = {
        style: mergedStyle,
        labels: labels,
        className: className
      };
      cache.set(argsStr, cacheValue);
      cache.set(className, cacheValue);
      return className;
    }

    // For testing only.
    css.getSheet = getSheet;
    return css;
  };
  var css = createCss();

  /* eslint-disable prefer-rest-params, prefer-spread */
  var create = function create(css$1) {
    if (css$1 === void 0) {
      css$1 = css;
    }
    return function createElement(type, props) {
      var args = arguments;
      if (props && props.css) {
        var className = css$1(props.css);
        var newProps = Object.assign({}, props);
        newProps.className = props.className ? props.className + " " + className : className;
        delete newProps.css;
        args[1] = newProps;
      }
      return React__namespace.createElement.apply(undefined, args);
    };
  };
  var jsx = create();

  Object.defineProperty(exports, 'ThemeProvider', {
    enumerable: true,
    get: function () { return theming.ThemeProvider; }
  });
  Object.defineProperty(exports, 'createTheming', {
    enumerable: true,
    get: function () { return theming.createTheming; }
  });
  Object.defineProperty(exports, 'useTheme', {
    enumerable: true,
    get: function () { return theming.useTheme; }
  });
  Object.defineProperty(exports, 'withTheme', {
    enumerable: true,
    get: function () { return theming.withTheme; }
  });
  Object.defineProperty(exports, 'SheetsRegistry', {
    enumerable: true,
    get: function () { return jss.SheetsRegistry; }
  });
  Object.defineProperty(exports, 'createGenerateId', {
    enumerable: true,
    get: function () { return jss.createGenerateId; }
  });
  exports.JssContext = JssContext;
  exports.JssProvider = JssProvider;
  exports.createJsx = create;
  exports.createUseStyles = createUseStyles$1;
  exports.default = withStyles;
  exports.jss = defaultJss$1;
  exports.jsx = jsx;
  exports.styled = configureStyled$1;
  exports.withStyles = withStyles;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=react-jss.js.map
